<?php
namespace Drupal\publishthis\Classes;

use \Drupal\publishthis\Classes\common\Publishthis_API_Common;
use \Drupal\Core\Logger\RfcLogLevel;

class Publishthis_API extends Publishthis_API_Common {

  /**
   * Publishthis_API constructor
   */
  function __construct() {    
    $config = \Drupal::config('publishthis.settings'); 
    $this->_api_url = PT_API_URL;
  }
  
  /*
  * Return token saved value
  */
  public function _get_token() {
    $config = \Drupal::config('publishthis.settings');  
    $saved_token = $config->get('pt_api_token'); 
    return $saved_token;
  }
  
  /*
  * Set dblogs for PublishThis
  */
  public function _log_message( $message, $level='' ) {
    if(!is_array($message)) {
      $message = ['message' => $message,  'status' => 'info'];
    }

    switch ($message['status']) {
      case 'warn':
        $severity = RfcLogLevel::WARNING;
        break;

      case 'error':
        $severity = RfcLogLevel::ERROR;
        break;
      
      case 'info':
      default:
        $severity = RfcLogLevel::INFO;
        break;
    }
    \Drupal::logger('publishthis')->log($severity, $message['message']);
  }

  /**
   * Returns Publishthis client info
   */
  public function get_client_info($params = []) {

    $params = $params + ['token' => $this->_get_token()];
    
    $url = $this->_compose_api_call_url('/client', $params);
    
    try {
      $response = $this->_request($url);
    } 
    catch (Exception $ex) {
      $this->_log_message($ex->getMessage(), '7');
      $response = null;
    }
    
    return $response;
  } 
  
  /**
   * Process API request
   */
  public function _request( $url, $return_errors = false ) {
    // check token setup
    $query_str = parse_url($url, PHP_URL_QUERY);
    parse_str($query_str, $query_params);
    if(empty($query_params['token'])) {
      return null;
    }

    // process request
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HEADER, false);
    $data = curl_exec($ch);

    // Check HTTP Code
    $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);

    // Close cURL Resource
    curl_close($ch);

    // check for failure
    if(!isset($data) || $status != 200) {      
      $message = [
        'message' => 'PublishThis API error URL ' . $url,
        'status' => 'error',
      ];
      $this->_log_message( $message, '2');
    }

    $json = '';
    try {
      $json = json_decode($data);

      if (!$json) {
        $message = [
          'message' => 'Inner JSON conversion error URL ' . $url,
          'status' => 'error',
        ];
        $this->_log_message( $message, '2');
      }

    } 
    catch (Exception $ex) {
      // try utf encoding it and then capturing it again.
      // we have seen problems in some wordpress/server installs where the json_decode
      //doesn't actually like the utf-8 response that is returned
      $message = [
        'message' => 'Issue in decoding the json ' . $ex->getMessage (),
        'status' => 'error',
      ];
      $this->_log_message($message, '2');

      try {
        $tmpBody = utf8_encode($data);
        $json = json_decode($tmpBody);
      } 
      catch (Exception $exc) {
        $message = [
          'message' => 'Issue in utf8 encoding and then decoding the json ' . $ex->getMessage (),
          'status' => 'error',
        ];
        $this->_log_message($message, '2');
      }
    }

    if (!$json) {
      $message = [
        'message' => 'JSON conversion error URL ' . $url,
        'status' => 'error',
      ];
      $this->_log_message($message, '2');
    }

    return $status == 200 ? $json->resp->data : null;

  }

  public function get_feed_templates($params = []) {
    $params = $params + ['token' => $this->_get_token()];

    $url = $this->_compose_api_call_url( '/publishtypes/posts', $params );

    try {
      $response = $this->_request($url);
      $templates = [];
      if($response->totalAvailable > 0) {
        foreach ($response->items as $key => $item) {
          $templates[$item->templateId] = $item->displayName;
        }
      }
      return $templates;
    } 
    catch ( Exception $ex ) {
      $this->_log_message( $ex->getMessage() );
    }
  }
}
